﻿/*******************************************************************************
 * Copyright 2010 Seapine Software, Inc. All rights reserved.
 * 
 * IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (“Seapine”)
 * in consideration of your agreement to the following terms, and your use, installation,
 * modification or redistribution of this Software constitutes acceptance of these terms.
 * If you do not agree with these terms, please do not use, install, modify or redistribute
 * this Software.
 * 
 * In consideration of your agreement to abide by the following terms, and subject to these
 * terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
 * this original Seapine software (the “Software”), to use, reproduce, modify and redistribute
 * the Software, with or without modifications, in source and/or binary forms; provided that
 * if you redistribute the Software in its entirety and without modifications, you must retain
 * this notice and the following text and disclaimers in all such redistributions of the Software.
 * Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
 * or promote products derived from the Software without specific prior written permission from
 * Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
 * implied, are granted by Seapine herein, including but not limited to any patent rights that
 * may be infringed by your derivative works or by other works in which the Software may be
 * incorporated.
 * 
 * The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
 * OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
 * 
 * IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
 * REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
 * UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
 * IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/



using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Ionic.Zip;
using System.Net;
using System.Net.Mail;

namespace TTEmailReport
{
    class Program
    {
        static void Main(string[] args)
        {
            if (args.Length == 2)
            {
                //Let's parse out the report name first
                string reportName = args[0];
                //Let's use the name for the html file and zip file
                string reportFile = reportName + ".html";
                string zipName = reportName + ".zip";

                //Now let's get the email address(es)
                string[] emailAddresses = args[1].Split(';');

                //Now let's read in the configuration file and set some variables
                Config config = new Config();
                if (config.Exists())
                {
                    Logger.WriteLine("Starting...");
                    //TestTrack variables
                    string ttUser = config.getUserNameFromFile();
                    string ttPassword = config.getPasswordFromFile();
                    string ttProjectName = config.getTTProjectName();
                    string ttURL = config.getTTURLFromFile();

                    if ((ttUser.Equals("")) || (ttProjectName.Equals("")) || (ttURL.Equals("")))
                    {
                        Logger.WriteLine("Missing information need to connect to TestTrack.");
                    }
                    else
                    {
                        //Email variables
                        string retAdress = config.getReturnAddress();
                        string displayName = config.getReturnDisplayName();

                        Logger.WriteLine("Connecting to TestTrack for report generation...");

                        //Let's initialize the SOAP object
                        ttsoapcgi ttSoap = new ttsoapcgi();

                        CProject[] ttProjects = ttSoap.getProjectList(ttUser, ttPassword);
                        CProject ttProject = null;

                        for (int i = 0; i < ttProjects.Length; i++)
                        {
                            if (ttProjects[i].database.name == ttProjectName)
                            {
                                ttProject = ttProjects[i];
                                break;
                            }
                        }
                        if (ttProject == null)
                        {
                            Logger.WriteLine("Could not connect to TestTrack. Project not found.");
                            return;
                        }
                        else
                        {
                            long ttCookie = ttSoap.ProjectLogon(ttProject, ttUser, ttPassword);
                            CDatabaseTable[] ttTables = ttSoap.getTableList(ttCookie);
                            foreach (CDatabaseTable ttTable in ttTables)
                            {
                                if (ttTable.name == "Report")
                                {
                                    //Let's get the columns for the Report table
                                    CTableColumn[] ttTableColumns = ttSoap.getColumnsForTable(ttCookie, ttTable.name);
                                    //Let's get the records in the Report table
                                    CRecordListSoap ttRecords = ttSoap.getRecordListForTable(ttCookie, ttTable.name, "", ttTableColumns);
                                    if (ttRecords != null)
                                    {
                                        CRecordRowSoap[] ttReports = ttRecords.records;
                                        foreach (CRecordRowSoap ttReport in ttReports)
                                        {
                                            //Column 1 in the 0 based array is the report name
                                            string rptName = ttReport.row[1].value;
                                            if (rptName.Equals(reportName))
                                            {
                                                //Column 0 in the 0 based array is the report ID, which we'll need to generate it
                                                long rptID = Convert.ToInt64(ttReport.row[0].value);

                                                bool fResult = false;
                                                try
                                                {
                                                    //Let's define an array to hold the report file(s)
                                                    CFile[] rptFiles = null;
                                                    Logger.WriteLine("Generating report...");
                                                    CReportRunResults results = ttSoap.getReportRunResultsByRecordID(ttCookie, rptID);
                                                    rptFiles = results.mFileList;
                                                    ZipFile zip = new ZipFile();
                                                    if (rptFiles != null && rptFiles.Length > 0)
                                                    {
                                                        //if the report only consists of one file we'll just attach it, otherwise we'll zip it up.
                                                        StreamWriter sw = new StreamWriter(reportFile, false);
                                                        //the first file in the file array will always be the html page. Other files will be images, java script files, etc...
                                                        sw.Write(System.Text.ASCIIEncoding.ASCII.GetString(rptFiles[0].mpFileData));
                                                        sw.Close();
                                                        for (int i = 1; i < rptFiles.Length; ++i)
                                                        {

                                                            zip.AddFile(reportFile);
                                                            // Write the images.
                                                            FileStream fs = File.Create(rptFiles[i].mstrFileName);
                                                            BinaryWriter bw = new BinaryWriter(fs);
                                                            bw.Write(rptFiles[i].mpFileData);
                                                            bw.Close();
                                                            fs.Close();
                                                            zip.AddFile(rptFiles[i].mstrFileName);

                                                        }
                                                        fResult = true;
                                                        zip.Save(zipName);
                                                        //Now to create the email
                                                        SmtpClient client = new SmtpClient("ex4.seapine.com");
                                                        MailAddress returnAddress = new MailAddress(retAdress, displayName, System.Text.Encoding.UTF8);
                                                        for (int i = 0; i < emailAddresses.Length; i++)
                                                        {
                                                            Logger.WriteLine("Generating email for " + emailAddresses[i]);
                                                            MailAddress sendToAddress = new MailAddress(emailAddresses[i]);
                                                            // Specify the message content.
                                                            MailMessage message = new MailMessage(returnAddress, sendToAddress);
                                                            message.Body = "Report sent to you automatically by TTEmailReport.";
                                                            message.Body += "\n\r";
                                                            message.Body += "Report generated on " + DateTime.Now.ToString();

                                                            message.Subject = reportName;
                                                            //Now we need to deal with attachments
                                                            if (rptFiles.Length == 1)
                                                            {
                                                                Attachment attFile = new Attachment(reportFile);
                                                                message.Attachments.Add(attFile);
                                                            }
                                                            else
                                                            {
                                                                Attachment attFile = new Attachment(zipName);
                                                                message.Attachments.Add(attFile);

                                                            }
                                                            client.Send(message);
                                                            message.Dispose();
                                                        }


                                                    }
                                                }
                                                catch (Exception fcc)
                                                {
                                                    Logger.WriteLine("General error detected: " + fcc.Message);
                                                }
                                            }
                                        }

                                    }
                                }
                            }

                            ttSoap.DatabaseLogoff(ttCookie);

                        }
                    }
                }
                else
                {
                    Logger.WriteLine("Configuration file not found!");
                }
            }
            else
            {
                Logger.WriteLine("Wrong number of parameters passed.  App only takes 2!");
            }

        }
    }
}
